@@ -46,7 +46,14 @@ function Print(guiAction, document, view) {
 
 Print.prototype = new File();
 
-Print.defaultPrinter = new QPrinter();
+Print.defaultPrinter = undefined;
+
+Print.getDefaultPrinter = function() {
+    if (isNull(Print.defaultPrinter)) {
+        Print.defaultPrinter = new QPrinter();
+    }
+    return Print.defaultPrinter;
+};
 
 Print.prototype.beginEvent = function() {
     File.prototype.beginEvent.call(this);
@@ -68,13 +75,7 @@ Print.prototype.beginEvent = function() {
     this.terminate();
 };
 
-/**
- * Prints the given document.
- *
- * \param pdfFile Name of PDF file
- * \param printerName Name of printer to print to or "" for default printer
- */
-Print.prototype.print = function(pdfFile, printerName) {
+Print.prototype.createPrinter = function(pdfFile, printerName) {
     var printer = undefined;
 
     // print directly to given printer or default printer:
@@ -90,6 +91,7 @@ Print.prototype.print = function(pdfFile, printerName) {
     }
 
     if (isNull(printer)) {
+        // create printer on the fly:
         printer = new QPrinter(QPrinter.HighResolution);
 
         if (isString(pdfFile)) {
@@ -167,10 +169,46 @@ Print.prototype.print = function(pdfFile, printerName) {
 
         if (Print.cancel===true) {
             printer.destroy();
-            return;
+            return undefined;
         }
     }
 
+    return printer;
+};
+
+/**
+ * Prints the given document.
+ *
+ * \param pdfFile Name of PDF file.
+ * \param printerName Name of printer to print to or "" for default printer.
+ * \param printer QPrinter object or undefined to create on the fly.
+ */
+Print.prototype.print = function(pdfFile, printerName) {
+    var printer = this.createPrinter(pdfFile, printerName);
+    if (isNull(printer)) {
+        qWarning("Print.prototype.print: no printer created");
+        return false;
+    }
+
+    var painter = new QPainter();
+    if (!painter.begin(printer)) {
+        printer.destroy();
+        return false;
+    }
+
+    this.printCurrentBlock(printer, painter);
+
+    painter.end();
+    printer.destroy();
+
+    return true;
+};
+
+/**
+ * Prints / exports the given block on a single or on multiple pages (if columns/rows are > 1).
+ */
+Print.prototype.printCurrentBlock = function(printer, painter) {
+
     // set background color of view to match printing preference:
     var bgColor = this.view.getBackgroundColor();
 
@@ -188,12 +226,6 @@ Print.prototype.print = function(pdfFile, printerName) {
         this.scene.regenerate();
     }
 
-    var painter = new QPainter();
-    if (!painter.begin(printer)) {
-        printer.destroy();
-        return false;
-    }
-
     // scale factor from drawing unit to mm:
     var unitScale = Print.getUnitScale(this.document);
 
@@ -225,16 +257,16 @@ Print.prototype.print = function(pdfFile, printerName) {
     }
 
     // iterate through all pages and print the appropriate area
-    var first = true;
+    var firstPage = true;
 
     var pages = Print.getPages(this.document);
     for (var i = 0; i < pages.length; ++i) {
         // if this is not the first page, add new page:
-        if (!first) {
+        if (!firstPage) {
             printer.newPage();
         }
         else {
-            first = false;
+            firstPage = false;
         }
 
         var c = Print.getBackgroundColor(this.document);
@@ -300,9 +332,6 @@ Print.prototype.print = function(pdfFile, printerName) {
         }
     }
 
-    painter.end();
-    printer.destroy();
-
     this.scene.setPixelSizeHint(previousPixelSizeHint);
 
     this.scene.setScreenBasedLinetypes(screenBasedLinetypes);
@@ -312,7 +341,7 @@ Print.prototype.print = function(pdfFile, printerName) {
     this.view.setBackgroundColor(bgColor);
     this.view.setPrinting(false);
 
-    return true;
+    //return true;
 };
 
 /**
@@ -329,6 +358,8 @@ Print.prototype.printPage = function(painter, rect) {
             new RVector(rect.x()+rect.width(), rect.y()+rect.height())
         )
     );
+
+    this.view.paintOverlay(painter);
 };
 
 /**
@@ -828,7 +859,7 @@ Print.getColorModeString = function(colorModeEnum) {
  * \return Default page rect of default printer in paper units.
  */
 Print.getDefaultPageRect = function(document, paperUnit) {
-    var r = Print.defaultPrinter.pageRect(QPrinter.Millimeter);
+    var r = Print.getDefaultPrinter().pageRect(QPrinter.Millimeter);
     if (isNull(paperUnit)) {
         if (!isNull(document)) {
             paperUnit = Print.getPaperUnit(document);
@@ -850,7 +881,7 @@ Print.getDefaultPageRect = function(document, paperUnit) {
  * \return Default paper rect of default printer in paper units.
  */
 Print.getDefaultPaperRect = function(document, paperUnit) {
-    var r = Print.defaultPrinter.paperRect(QPrinter.Millimeter);
+    var r = Print.getDefaultPrinter().paperRect(QPrinter.Millimeter);
     if (isNull(paperUnit)) {
         if (!isNull(document)) {
             paperUnit = Print.getPaperUnit(document);
@@ -1001,7 +1032,7 @@ Print.setOffset = function(di, offset) {
  * \return Paper size in MM as QSizeF object.
  */
 Print.getPaperSizeMM = function(document) {
-    //var defaultPaperSizeMM = Print.defaultPrinter.paperSize(QPrinter.Millimeter);
+    //var defaultPaperSizeMM = Print.getDefaultPrinter().paperSize(QPrinter.Millimeter);
     var wMM = RUnit.convert(Print.getPaperWidth(document), Print.getPaperUnit(document), RS.Millimeter);
     var hMM = RUnit.convert(Print.getPaperHeight(document), Print.getPaperUnit(document), RS.Millimeter);
     return new QSizeF(wMM, hMM);
@@ -1012,7 +1043,7 @@ Print.getPaperSizeMM = function(document) {
  * or QPrinter.
  */
 Print.getDefaultPaperSizeMM = function() {
-    var defaultPaperSizeMM = Print.defaultPrinter.paperSize(QPrinter.Millimeter);
+    var defaultPaperSizeMM = Print.getDefaultPrinter().paperSize(QPrinter.Millimeter);
 
     var dwMM = defaultPaperSizeMM.width();
     var dhMM = defaultPaperSizeMM.height();
